package com.syntevo.plugin.jira.commit.workflow;

import java.awt.*;
import java.awt.event.*;
import java.util.List;
import javax.swing.*;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.plugin.jira.transport.*;
import com.syntevo.q.gui.layout.*;

/**
 * @author syntevo GmbH
 */
final class JiraResolveIssueDialog extends AbstractOkCancelDialog {

	// Static =================================================================

	public static JiraResolveIssueDialog getInstance(JiraResolvablePacket packet, GuiSpacings spacings) {
		final List<JiraVersion> unreleasedVersions = packet.getUnreleasedVersions();
		if (unreleasedVersions == null || unreleasedVersions.isEmpty()) {
			return null;
		}

		final JiraIssue issue = packet.getIssue();
		String summary = issue.getSummary();
		if (summary.length() > 50) {
			summary = summary.substring(0, 47) + "...";
		}

		return new JiraResolveIssueDialog("Do you want to resolve issue " + issue.getKey() + ": " + summary + "?", unreleasedVersions, issue.getFixVersions(), spacings);
	}

	// Fields =================================================================

	private final GuiSpacings spacings;
	private final JLabel messageLabel;
	private final JComboBox fixVersionComboBox;
	private final JRadioButton dontResolveButton;
	private final JRadioButton resolveButton;

	// Setup ==================================================================

	private JiraResolveIssueDialog(String message, List<JiraVersion> unreleasedVersions, List<JiraVersion> fixVersions, GuiSpacings spacings) {
		this.spacings = spacings;

		messageLabel = new JLabel(message);

		resolveButton = new JRadioButton("Mark as resolved in version");

		dontResolveButton = new JRadioButton("Don't mark as resolved");
		dontResolveButton.setSelected(true);

		final ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(resolveButton);
		buttonGroup.add(dontResolveButton);

		fixVersionComboBox = new JComboBox();
		for (JiraVersion version : unreleasedVersions) {
			fixVersionComboBox.addItem(version);
		}

		for (int index = 0; index < fixVersionComboBox.getItemCount(); index++) {
			final JiraVersion version = (JiraVersion)fixVersionComboBox.getItemAt(index);
			if (fixVersions.contains(version)) {
				fixVersionComboBox.setSelectedIndex(index);
			}
		}

		final ListCellRenderer nativeRenderer = fixVersionComboBox.getRenderer();
		fixVersionComboBox.setRenderer(new ListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				return nativeRenderer.getListCellRendererComponent(list, value != null ? ((JiraVersion)value).getName() : null, index, isSelected, cellHasFocus);
			}
		});

		final ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateEnabledStates();
			}
		};
		resolveButton.addActionListener(actionListener);
		dontResolveButton.addActionListener(actionListener);

		updateEnabledStates();
	}

	// Implemented ============================================================

	public String getTitle() {
		return "Resolve JIRA Issue";
	}

	public JComponent createComponent(Container parent) {
		final JComponent panel = new JPanel();
		parent.add(panel);

		final TableGridLayoutFactory layout = new TableGridLayoutFactory(panel);
		layout.addColumnNoStretch();
		layout.addColumnNoStretch(spacings.getXUnrelated());
		layout.addColumnStretchOnResize();
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addComponent(messageLabel, 0, 0, 1, 3);
		layout.addComponent(resolveButton, 2, 0);
		layout.addComponent(fixVersionComboBox, 2, 2);
		layout.addComponent(dontResolveButton, 4, 0, 1, 3, true, true, true, false);
		return panel;
	}

	@Override
	public void dialogShowing() {
		super.dialogShowing();
		if (fixVersionComboBox.isEnabled()) {
			fixVersionComboBox.requestFocusInWindow();
		}
		else {
			dontResolveButton.requestFocusInWindow();
		}
	}

	// Accessing ==============================================================

	public boolean isResolve() {
		return resolveButton.isSelected();
	}

	public JiraVersion getFixVersion() {
		return (JiraVersion)fixVersionComboBox.getSelectedItem();
	}

	// Utils ==================================================================

	private void updateEnabledStates() {
		fixVersionComboBox.setEnabled(resolveButton.isSelected());
	}
}
